from dqc_wrapper.test06_valid_ranges import ValidRangesTest
from dqc_wrapper.unit_tests.conftest import dirs
from pathlib import Path
import xarray as xr
import numpy as np
import pandas as pd


def test_get_stats_from_hist():
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '',
                           0, '')
    ds = xr.open_dataset(dirs.nc_file)
    x, b = np.histogram(ds['tas'].data, bins=np.arange(10, 26))
    m, dev, med, q1, q3, IQD, skew = test.get_stats_from_hist(x, b[:-1])
    assert ds['tas'].size/16 <= m and m <= ds['tas'].size/14
    assert dev > 0
    assert skew < 1


def test_get_basic_stats():
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '', 0, '')
    ds = xr.open_dataset(dirs.nc_file)
    # random values between 10 and 25
    min, max, mean, std = test.get_basic_stats(ds['tas'].data)
    assert min >= 10
    assert max <= 25
    assert mean >= min and mean <= max
    assert std > 0


def test_load_datafile():
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '',
                           0, '')
    ds, arr = test.load_datafile(dirs.nc_file)
    assert ds
    assert len(arr) == 5*320*160


def test_get_hist_hadley():
    df_hist = pd.DataFrame(index=['N_polar'], columns=np.arange(15))
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '',
                           0, '')
    ds = xr.open_dataset(dirs.nc_file)
    test.get_hist_hadley(df_hist, np.arange(10, 26), ds, 0, 60, 90, 'N_polar',
                         'lat')
    assert df_hist.shape == (1, 15)


def test_get_bins_init():
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '',
                           0, '')
    hist = test.get_bins_init(dirs.nc_file)
    assert len(hist) == 2001


def test_get_stats_files():
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '',
                           0, '')
    stats, hist = test.get_stats_files()
    assert stats == dirs.csv
    assert hist == dirs.csv

def test_check_valid_ranges():
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', '', '', '',
                           0, '')
    res, msg = test.check_valid_ranges()
    assert res == "ok"
    # Min and max outside range
    test = ValidRangesTest(dirs.log_dir, '', [dirs.nc_file], 'tas', 100, 200,
                           '', 0, '')
    res, msg = test.check_valid_ranges()
    assert res == "err"
    # Deleting generated CSV
    Path(dirs.test_dir, "sample").unlink()

test_check_valid_ranges()